/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.common;

import java.net.InetAddress;

import java.io.*;

/**
 * Represents a host computer running iTunes for the remote to connect to
 *
 */
public class Host implements Serializable {

	private String hostName = null;
	private InetAddress addr = null;
	private boolean rememberPassword = false;
	private String password = null;
	
	public Host(String hostName, InetAddress addr, String password, 
	        boolean rememberPassword) {
		this.hostName = hostName;
		this.addr = addr;
		this.password = password;
		this.rememberPassword = rememberPassword;
	}
	
	public Host(String hostName, InetAddress addr) {
	    this(hostName,addr,null,false);
	}
	
	private void writeObject(ObjectOutputStream out) throws IOException {
	    // TODO if any changes are made to this class then this method must be changed
	    out.writeObject(hostName);
	    out.writeObject(addr);
        out.writeObject(new Boolean(rememberPassword));
	    if(rememberPassword)
	        out.writeObject(password);
	    else
	        out.writeObject("");
	}
	
	private void readObject(ObjectInputStream in) throws IOException, 
								ClassNotFoundException {
	    // TODO if any changes are made to this class then this method must be changed
	    hostName = (String)in.readObject();
	    addr = (InetAddress)in.readObject();
	    rememberPassword = ((Boolean)in.readObject()).booleanValue();
	    password = (String)in.readObject();
	}
	
    public void setPassword(String password) {
        this.password = password;
    }
    
    public void setRememberPassword(boolean rememberPassword) {
        this.rememberPassword = rememberPassword;
    }
    
	public void setHostName(String _hostName) {
	    hostName = _hostName;
	}
	
	public void setAddress(InetAddress _addr) {
	    addr = _addr;
	}
	
    public boolean isRememberPassword() {
        return rememberPassword;
    }

	public InetAddress getAddress() {
		return addr;
	}
	
	public String getHostName() {
		return hostName;
	}
	
    public String getPassword() {
        return password;
    }
	
	public String toString() {
		return hostName + " @ " + addr.getHostAddress();
	}
	
	public boolean equals(Object o) {
		if( !(o instanceof Host) )
			return false;
		Host other = (Host)o;
		return other.getAddress().equals(addr);
	}
	

}
