/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.common;

import java.io.Serializable;

/**
 * The track class encapsualtes one track
 * 
 * It provides methods for getters and setters
 * as well as methods for transmitting and receiving
 * across the network
 *
 */
public class Track implements Serializable {

	private String title;
	private String artist;
	private String album;
	private String length;  		// in ms
	
	private int sourceID;
	private int playlistID;
	private int trackID;
	private int databaseID;
	
	
	public Track() { }
	
	public Track(String title, String artist, String album,
			String length, int sourceID, int playlistID,
			int trackID, int databaseID) {
		this.title = title;
		this.artist = artist;
		this.album = album;
		this.length = length;
		this.sourceID = sourceID;
		this.playlistID = playlistID;
		this.trackID = trackID;
		this.databaseID = databaseID;
	}
	
	public Track(String title, String artist, String album,
			String length, String sourceID, String playlistID,
			String trackID, String databaseID) throws NumberFormatException {
		this.title = title;
		this.artist = artist;
		this.album = album;
		this.length = length;
		this.sourceID = Integer.parseInt(sourceID);
		this.playlistID = Integer.parseInt(playlistID);
		this.trackID = Integer.parseInt(trackID);
		this.databaseID = Integer.parseInt(databaseID);
	}
	
	public Track(String s) throws NumberFormatException {
		String[] parts = s.split("\n");
		title = parts[0];
		artist = parts[1];
		album = parts[2];
		length = parts[3];
		sourceID = Integer.parseInt(parts[4]);
		playlistID = Integer.parseInt(parts[5]);
		trackID = Integer.parseInt(parts[6]);
		databaseID = Integer.parseInt(parts[7]);
	}
	
	public Track(byte[] buf) {
		this(new String(buf));
	}
	
	public boolean equals(Object other) {
		if(!(other instanceof Track))
			return false;
		Track otherTrack = (Track)other;
		if(otherTrack.getDatabaseId() == databaseID)
			return true;
		return false;
	}
	
	
	public String toString() {
		String nl = "\n";
		return title + nl + artist + nl + album + nl + length + nl + 
		sourceID + nl + playlistID + nl + trackID + nl + databaseID;
	}
	
	public String toListString() {
		String d = ", ";
		return title + d + artist + d + album + d + length;
	}
	
	public byte[] getBytes() {
		return toString().getBytes();
	}
	
	public String getAlbum() {
		return album;
	}
	public void setAlbum(String album) {
		this.album = album;
	}
	public String getArtist() {
		return artist;
	}
	public void setArtist(String artist) {
		this.artist = artist;
	}
	public int getDatabaseId() {
		return databaseID;
	}
	public void setDatabaseId(int id) {
		databaseID = id;
	}
	
	public String getLength() {
		return length;
	}
	
	public void setLength(String length) {
		this.length = length;
	}
	
	public String getTitle() {
		return title;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	public int getPlaylistID() {
		return playlistID;
	}
	public void setPlaylistID(int playlistID) {
		this.playlistID = playlistID;
	}
	public int getSourceID() {
		return sourceID;
	}
	public void setSourceID(int sourceID) {
		this.sourceID = sourceID;
	}
	public int getTrackID() {
		return trackID;
	}
	public void setTrackID(int trackID) {
		this.trackID = trackID;
	}
}
