/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local;


import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketTimeoutException;

import walstad.itunesremote.common.*;


/**
 * Replies to remotes so that they can 
 * then connect
 */
public class Discoverer extends Thread {

	private volatile boolean keepRunning = true;
	private volatile Exception error = null;
	
	public void end() {
		keepRunning = false;
	}
	
	public boolean wasError() {
		return (error != null);
	}
	
	public Exception getError() {
		return error;
	}
	
	public void run() {
		DatagramSocket ds = null;
		byte[] buf = new byte[Common.BUFLENGTH];
		DatagramPacket recvPacket = new DatagramPacket(buf,buf.length);
		DatagramPacket sendPacket = null;
		
		try {
			ds = new DatagramSocket(Common.ANNOUNCER_PORT);
			ds.setSoTimeout(100);
		
			while(keepRunning) {
				try {
					ds.receive(recvPacket);
				}catch(SocketTimeoutException e) {
					continue;
				}
				if(Tools.areEqual(recvPacket.getData(),Common.REMOTE_ID)) {
					byte[] hostName;
					Properties p = Main.getProperties();
					if(p == null || p.getHostName() == null)
						hostName = "default_name".getBytes();
					else
						hostName = p.getHostName().getBytes();
					
					byte[] data = new byte[hostName.length + Common.COMP_ID.length];
					for(int i=0;i<Common.COMP_ID.length;i++)
						data[i] = Common.COMP_ID[i];
					for(int i=Common.COMP_ID.length;i<data.length;i++)
						data[i] = hostName[i-Common.COMP_ID.length];
						
					sendPacket = new DatagramPacket(data,data.length);
					sendPacket.setAddress(recvPacket.getAddress());
					sendPacket.setPort(recvPacket.getPort());
					ds.send(sendPacket);
				}
			}
			
		}catch(Exception e) {
			e.printStackTrace();
			error = e;
		}finally {
			try { ds.close(); } catch(Exception e) {}
		}
	}

}
