/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local;

import org.jdesktop.jdic.tray.*;

import org.bouncycastle.crypto.params.KeyParameter;
import org.bouncycastle.crypto.macs.HMac;
import org.bouncycastle.crypto.digests.SHA256Digest;

import walstad.itunesremote.common.Debugger;
import walstad.itunesremote.common.security.Tools;
import walstad.itunesremote.local.gui.GuiMainFrame;

import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.*;

import java.util.List;
import java.util.ArrayList;



/**
 * The entry point into the host portion of the remote control
 */
public class Main implements ActionListener {
	
	private static Properties properties = null;
	
	private static GuiMainFrame mainFrame = null;
	
	private static final String HELP_URL = "http://itremote.sourceforge.net/help.php";
	private static final String ABOUT_MSG = "iTunes Remote version 1.0 Beta\n" +
			"Allows you to use a PDA or other wireless device to remotely " +
			"control iTunes.\n" +
			"Developed by Chris Walstad.";
	
	private static boolean keepRunning = true;
	
	private static List connectionListeners = new ArrayList();
	
	private static TrayIcon trayIcon;
	
	private static HMac hmac;
	
	public static void main(String[] args) {		
		setupGui();
		run();
	}
	
	public synchronized static HMac getHMac() {
	    return hmac;
	}
	
	public synchronized static void setPassword(String password) {
	    hmac.init(new KeyParameter(Tools.stringToKey(password)));
	}
	
	public static JFrame getMainFrame() {
		return mainFrame;
	}
	
	private static void setConnectionStatus(String status) {
		ConnectionStatusChangedEvent evt = new ConnectionStatusChangedEvent(status);
		for(int i=0;i<connectionListeners.size();i++) {
			((ConnectionStatusChangedEventListener)connectionListeners.get(i)).
				connectionStatusChanged(evt);
		}
		trayIcon.setCaption("iTunes Remote: " + status);
	}
	
	public static void addConnectionStatusEventListener(ConnectionStatusChangedEventListener l) {
		connectionListeners.add(l);
	}
	
	private static void run() {
		Player player = new Player();
		while(keepRunning) {
			Discoverer discoverer = new Discoverer();
			setConnectionStatus("Not connected");
			discoverer.start();
			Connection connection = new Connection();
			if(!connection.connect()) {
				Debugger.println("Could not connect");
				discoverer.end();
				connection.cleanup();
				try {
					Thread.sleep(100);
				}catch(InterruptedException e) {}
				continue;
			}
			setConnectionStatus("Connected");
			discoverer.end();
			Receiver receiver = new Receiver(player,connection);
			receiver.run();
			// here we have disconnected, so clean up resources
			connection.cleanup();
		}
	}
	
	private static void setupGui() {
		Toolkit kit = Toolkit.getDefaultToolkit();
		JFrame.setDefaultLookAndFeelDecorated(false);
		
		// Setup the SystemTray stuff
		SystemTray sysTray = SystemTray.getDefaultSystemTray();
		JPopupMenu menu = new JPopupMenu();
		JMenuItem exitMenuItem = new JMenuItem("Exit");
		exitMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});
		JMenuItem aboutMenuItem = new JMenuItem("About");
		aboutMenuItem.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        JOptionPane.showMessageDialog(mainFrame,ABOUT_MSG,"About",
		                JOptionPane.INFORMATION_MESSAGE);
		    }
		});
		JMenuItem helpMenuItem = new JMenuItem("Help");
		helpMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " + HELP_URL);
				}catch(Exception ex) {
					Debugger.println("Unable to open browser");
				}
			}
		});
		menu.add(helpMenuItem);
		menu.add(aboutMenuItem);
		menu.add(exitMenuItem);
		
		trayIcon = new TrayIcon(new ImageIcon(kit.getImage("res/icon.PNG")),
				"iTunes Remote",menu);
		trayIcon.addActionListener(new Main());
		sysTray.addTrayIcon(trayIcon);
		
		properties = Properties.readFromFile();
		if(properties == null) {
			properties = Properties.getDefaultProperties();
			try {
				properties.writeToFile();
			}catch(Exception e) {
				Debugger.println("Unable to write properties file");
			}
		}
		hmac = new HMac(new SHA256Digest());
		if(properties.isPasswordRequired())
		    hmac.init(new KeyParameter(Tools.stringToKey(properties.getPassword())));
		
		mainFrame = new GuiMainFrame();
	}
	
	public static Properties getProperties() {
		return properties;
	}
	
	public void actionPerformed(ActionEvent e) {
		boolean isVisible = mainFrame.isVisible();
		mainFrame.setVisible(!isVisible);
	}
	
	public void end() {
		keepRunning = false;
	}
}
