/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local;


import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;

import walstad.itunesremote.common.*;

/**
 * Methods for controlling and receiving information
 * from the iTunes player
 */
public class Player {
	
	private Track currentTrack;
	private Runtime runtime;
	
	public Player() {
		runtime = Runtime.getRuntime();
	}
	
	public int toggleVisualsEnabled() {
	    try {
	        execute("scripts/toggleVisuals.js",3);
	    }catch(IOException e) {
	        e.printStackTrace();
	        return Common.UNEXPECTED_ERROR;
	    }
	    return Common.SUCCESS;
	}
	
	public SearchResult search(SearchQuery sq) {
		SearchResult sr = new SearchResult(sq.getQuery(),null);
		String res;
		try {
			res = execute("scripts/searchAll.js " + "\"" +
					sq.getQuery() + "\"",10);
		}catch(IOException e) {
			e.printStackTrace();
			return sr;
		}
		if(res == null || res.length() == 0)
			return sr;
		String[] strings = res.split("\t");
		Track[] tracks = new Track[strings.length];
		for(int i=0;i<tracks.length;i++) {
			tracks[i] = new Track(strings[i]);
		}
		sr.setTracks(tracks);
		return sr;
	}
	
	public int play(int source, int playlist, int track, int database) {
		try {
			execute("scripts/play.js " + source + " " + playlist + " "
					+ track + " " + database,3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int play(Track t) {
		return play(t.getSourceID(),t.getPlaylistID(),t.getTrackID(),t.getDatabaseId());
	}
	
	public int stop()  {
		try {
			execute("scripts/stop.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int volumeUp() {
		try {
			execute("scripts/volumeUp.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int volumeDown() {
		try {
			execute("scripts/volumeDown.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int fastForward() {
		try {
			execute("scripts/fastForward.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int rewind() {
		try {
			execute("scripts/rewind.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int mute() {
		try {
			execute("scripts/mute.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int quit() {
		try {
			execute("scripts/stop.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int nextTrack() {
		try {
			execute("scripts/nextTrack.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int previousTrack() {
		try {
			execute("scripts/prevTrack.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int playPause() {
		try {
			execute("scripts/playPause.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int resume() {
		try {
			execute("scripts/resume.js",3);
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	/**
	 * @return the current track or null if no track or has
	 * not changed since last called
	 */
	public Track getCurrentTrack() throws IOException {
		Track track = new Track(execute("scripts/getCurrentTrackInfo.js",3));
		if(currentTrack != null && track.equals(currentTrack))
			return null;
		
		currentTrack = track;
		return track;
	}
	
	
	/**
	 * Executes the specified command with the given timeout
	 * @param cmd command to execute
	 * @param timeout timeout in seconds
	 * @return the string that the script returns
	 * @throws IOException if an unexpected error occurs
	 */
	private String execute(String cmd, int timeout) throws IOException {
		Process p = runtime.exec("Wscript.exe " + cmd + " //T:" + timeout);
		
		BufferedReader rdr = new BufferedReader(
				new InputStreamReader(p.getInputStream()));
		
		String ret = "";
		String line;
		while((line = rdr.readLine()) != null) {
			ret += line;
			ret += "\n";
		}
		
		return ret;
	}
	
}
