/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.local.gui;

import javax.swing.*;

import walstad.itunesremote.common.*;
import walstad.itunesremote.common.gui.SelectAllFocustListener;
import walstad.itunesremote.local.Main;
import walstad.itunesremote.local.Properties;

import java.awt.*;
import java.awt.event.*;


/**
 * The main panel for the host GUI
 *
 */
public class GuiMainPanel extends JPanel implements ActionListener {
	
	private JLabel nameLabel;
	private JTextField nameField;
	private JLabel oldPasswordLabel;
	private JPasswordField oldPasswordField;
	private JLabel newPasswordLabel;
	private JPasswordField newPasswordField;
	private JLabel confirmPasswordLabel;
	private JPasswordField confirmPasswordField;
	private JButton changeButton;
	private JButton cancelButton;
	private JButton okButton;
	private JCheckBox usePasswordCheckBox;
	private Properties properties;
	private JCheckBox displayOnStartupCheckBox;
	
	public GuiMainPanel() {
		super(new GridBagLayout());
		setup();
	}
	
	private void setPasswordEnabled(boolean b) {
	    newPasswordLabel.setEnabled(b);
	    confirmPasswordLabel.setEnabled(b);
	    newPasswordField.setEnabled(b);
	    confirmPasswordField.setEnabled(b);
	}
	
	private void setup() {
		properties = Main.getProperties();
		
		nameLabel = new JLabel("Host Name");
		nameLabel.setToolTipText("The Host name is used to identify your computer " + 
				"to remotes");
		nameField = new JTextField(properties.getHostName());
		oldPasswordLabel = new JLabel("Enter Old Password:");
		newPasswordLabel = new JLabel("New Password");
		confirmPasswordLabel = new JLabel("Confirm Password");
		oldPasswordField = new JPasswordField();
		newPasswordField = new JPasswordField();
		confirmPasswordField = new JPasswordField();
		changeButton = new JButton("Change");
		cancelButton = new JButton("Cancel");
		okButton = new JButton("OK");
		usePasswordCheckBox = new JCheckBox("Use a Password",properties.isPasswordRequired());
		usePasswordCheckBox.addActionListener(this);
		setPasswordEnabled(properties.isPasswordRequired());
		oldPasswordField.setToolTipText("You should choose a good password to prevent " +
			"other people from connecting to your computer with their remote");
		displayOnStartupCheckBox = new JCheckBox("Show on Startup",
		        properties.isDisplayOnStartup());
		displayOnStartupCheckBox.setToolTipText("If this box is checked then " +
				"this window will be displayed when iTunes Remote launches");
		displayOnStartupCheckBox.addActionListener(this);
		
		// Alignment
		oldPasswordLabel.setVerticalAlignment(SwingConstants.TOP);
		
		// Fonts
		Font font = new Font("TimesRoman",Font.PLAIN,12);
		setFont(font);
		nameField.setFont(font);
		oldPasswordLabel.setFont(font);
		newPasswordLabel.setFont(font);
		confirmPasswordLabel.setFont(font);
		oldPasswordField.setFont(font);
		newPasswordField.setFont(font);
		confirmPasswordField.setFont(font);
		usePasswordCheckBox.setFont(font);
		
		GridBagConstraints gbc = new GridBagConstraints();
		
		setLayout(new GridBagLayout());
		
		Tools.setGBC(gbc,0,0,2,1,100,100,GridBagConstraints.BOTH);
		add(nameLabel,gbc);
		
		Tools.setGBC(gbc,0,1,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(nameField,gbc);
		
		Tools.setGBC(gbc,0,3,2,1,0,0,GridBagConstraints.HORIZONTAL);
		add(displayOnStartupCheckBox,gbc);
		
		Tools.setGBC(gbc,0,0,2,1,0,0,GridBagConstraints.HORIZONTAL);
		add(oldPasswordLabel,gbc);
		
		Tools.setGBC(gbc,0,1,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(oldPasswordField,gbc);
		
		Tools.setGBC(gbc,0,2,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(okButton,gbc);
		
		Tools.setGBC(gbc,0,2,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(usePasswordCheckBox,gbc);
		
		Tools.setGBC(gbc,0,3,1,1,0,0,GridBagConstraints.NONE);
		add(newPasswordLabel,gbc);
		
		Tools.setGBC(gbc,1,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(newPasswordField,gbc);
		
		Tools.setGBC(gbc,0,4,1,1,0,0,GridBagConstraints.NONE);
		add(confirmPasswordLabel,gbc);
		
		Tools.setGBC(gbc,1,4,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(confirmPasswordField,gbc);
		
		Tools.setGBC(gbc,0,5,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(changeButton,gbc);
		
		Tools.setGBC(gbc,1,5,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(cancelButton,gbc);
		
		showMainScreen(properties.getPassword().equals(""));
		
		oldPasswordField.addActionListener(this);
		okButton.addActionListener(this);
		changeButton.addActionListener(this);
		cancelButton.addActionListener(this);
		confirmPasswordField.addActionListener(this);
		newPasswordField.addActionListener(this);
		nameField.addActionListener(this);
		
		okButton.setMnemonic('o');
		changeButton.setMnemonic('c');
		cancelButton.setMnemonic('n');
		
		newPasswordField.addFocusListener(new SelectAllFocustListener(newPasswordField));
		confirmPasswordField.addFocusListener(new SelectAllFocustListener(confirmPasswordField));
		oldPasswordField.addFocusListener(new SelectAllFocustListener(oldPasswordField));
	}
	
	private static final String NAME_ERROR_MESSAGE = "The name must be a non-empty" +
						" string with no more than 20 characters";
	private static final String PASSWORD_ERROR_MESSAGE = "Your password must be non-empty";
	
	private void handleChangeButtonActionEvent(ActionEvent evt) {
	    Properties p = Main.getProperties();
	    
	    String name = nameField.getText();
	    String newPassword = new String(newPasswordField.getPassword());
	    String confirmPassword = new String(confirmPasswordField.getPassword());
	    
	    
	    // Check errors
		if(name == null || name.equals("") || name.length() > 20) {
			showErrorDialog("Invalid Name",NAME_ERROR_MESSAGE);
			return;
		}		
		
		if(usePasswordCheckBox.isSelected()) {
		    if(!newPassword.equals(confirmPassword)) {
		        showErrorDialog("Passwords do not match","The confirm password is not equal to the new password.  Please retype your new password.");
		        return;
		    }
		    if(newPassword.equals("")) {
		        showErrorDialog("Invalid Password","Invalid password entered.\n  Password may not be empty");
		        return;
		    }
		}
		
		// apply changes
		p.setHostName(nameField.getText());
		if(usePasswordCheckBox.isSelected()) {
		    p.setPasswordRequired(true);
		    p.setPassword(newPassword);
		    Main.setPassword(newPassword);
		}else {
		    p.setPasswordRequired(false);
		    p.setPassword("");
		    Main.setPassword("");
		}
		
	    newPasswordField.setText("");
	    confirmPasswordField.setText("");
	    
	    if(p.isPasswordRequired())
	        showMainScreen(false);
		
		// Write the properties file
		try {
			p.writeToFile();
		}catch(Exception e) {
			showErrorDialog("IO Error","Could not write the properies file!\nPreferences will not be saved.");
		}
	}
	
	private void handleCancelButtonActionEvent(ActionEvent evt) {
	    Properties p = Main.getProperties();
	    nameField.setText(p.getHostName());
	    oldPasswordField.setText("");
	    newPasswordField.setText("");
	    confirmPasswordField.setText("");
	    usePasswordCheckBox.setSelected(p.isPasswordRequired());
	    setPasswordEnabled(p.isPasswordRequired());
	    showMainScreen(false);	    
	}
	
	private void handleUsePasswordCheckBoxActionEvent(ActionEvent evt) {
	    setPasswordEnabled(usePasswordCheckBox.isSelected());
	}
	
	public void actionPerformed(ActionEvent evt) {
		if(evt.getSource() == cancelButton) {
		    handleCancelButtonActionEvent(evt);
		}else if(evt.getSource() == changeButton || evt.getSource() == nameField ||
		        evt.getSource() == newPasswordField || 
		        evt.getSource() == confirmPasswordField) {
			handleChangeButtonActionEvent(evt);
		}else if(evt.getSource() == usePasswordCheckBox) {
		    handleUsePasswordCheckBoxActionEvent(evt);
		}else if(evt.getSource() == okButton || evt.getSource() == oldPasswordField) {
		    handleOKButtonActionEvent(evt);
		}else if(evt.getSource() == displayOnStartupCheckBox) {
		    handleDisplayOnStartupCheckBoxActionEvent(evt);
		}else {
			Debugger.println("Got unrecognized actionPerformed()");
		}
	}
	
	
	private void handleDisplayOnStartupCheckBoxActionEvent(ActionEvent evt) {
	    properties.setDisplayOnStartup(displayOnStartupCheckBox.isSelected());
	    try {
	        properties.writeToFile();
	    }catch(java.io.IOException e) {
	        showErrorDialog("Cannot Save Preferences",
	                "Unable to write preferences. Detailed information follows:\n" +
	                e.toString());
	    }
	}
	
	private void showMainScreen(boolean b) {
        okButton.setVisible(!b);
	    oldPasswordLabel.setVisible(!b);
	    oldPasswordField.setVisible(!b);
	    displayOnStartupCheckBox.setVisible(!b);
	    cancelButton.setVisible(b);
	    changeButton.setVisible(b);
	    confirmPasswordField.setVisible(b);
	    confirmPasswordLabel.setVisible(b);
	    nameField.setVisible(b);
	    nameLabel.setVisible(b);
	    newPasswordField.setVisible(b);
	    newPasswordLabel.setVisible(b);
	    usePasswordCheckBox.setVisible(b);
	}
	
	private void handleOKButtonActionEvent(ActionEvent evt) {
	    String password = oldPasswordField.getText();
	    if(password.equals(properties.getPassword())) {
	        newPasswordField.setText(properties.getPassword());
	        confirmPasswordField.setText(properties.getPassword());
	        showMainScreen(true);
	    }else {
	        showErrorDialog("Incorrect Password","The password you typed is incorrect.\nPlease try again.");
	    }
	    oldPasswordField.setText("");
	}
	
	private void showErrorDialog(String title,String msg) {
	    JOptionPane.showMessageDialog(Main.getMainFrame(), msg, title, 
	            JOptionPane.ERROR_MESSAGE);
	}
	
}