/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import java.io.IOException;

import java.util.List;
import java.util.ArrayList;

import walstad.itunesremote.common.*;



/**
 * Broadcasts the remote's location
 */
public class Announcer extends Thread {

	private int timeout;
	private volatile Exception exception;
	private volatile boolean keepRunning = true;
	private List discovered = new ArrayList();
	private volatile boolean isRunning = false;
	
	
	// listeners to announcer events
	private List discoveredListeners = new ArrayList();
	private List finishedListeners = new ArrayList();
	
	/**
	 * @param timeout in millseconds
	 */
	public Announcer(int timeout) {
		super();
		this.timeout = timeout;
	}
	
	/**
	 * @return a List of InetAddress objects that are discovered
	 */
	public List getDiscovered() {
		return discovered;
	}
	
	public boolean wasError() {
		return (exception != null);
	}
	
	public Exception getError() {
		return exception;
	}
	
	public void end() {
		keepRunning = false;
	}
	
	public boolean isRunning() {
		return isRunning;
	}
	
	public void run() {
		isRunning = true;
		long startTime = System.currentTimeMillis();
		DatagramSocket ds = null;
		try {
			byte[] recvBuf = new byte[Common.MAXBUF];
			
			ds = new DatagramSocket();
			ds.setSoTimeout(500);			
			DatagramPacket sendPacket = new DatagramPacket(Common.REMOTE_ID,Common.REMOTE_ID.length,
					InetAddress.getByName("255.255.255.255"),Common.ANNOUNCER_PORT);
			DatagramPacket receivePacket = new DatagramPacket(recvBuf,
					recvBuf.length);
			
			while( (System.currentTimeMillis() - startTime) < timeout &&
					keepRunning) {
				ds.send(sendPacket);
				try {
					ds.receive(receivePacket);
				}catch(IOException e) {
					continue;
				}
				byte[] data = receivePacket.getData();
				byte[] idBuf = new byte[Common.COMP_ID.length];
				byte[] hostNameBuf = new byte[data.length-idBuf.length];
				for(int i=0;i<idBuf.length;i++)
					idBuf[i] = data[i];
				for(int i=0;i<hostNameBuf.length;i++)
					hostNameBuf[i] = data[idBuf.length + i];
				String name = new String(hostNameBuf);
				name = name.trim();
				Host newHost = new Host(name,receivePacket.getAddress());
				if(Tools.areEqual(idBuf,Common.COMP_ID) &&
						!discovered.contains(newHost)) {
					discovered.add(newHost);
					notifyDiscoveredHostsEventListeners(newHost);
				}
			}	
		}catch(Exception e) {
			e.printStackTrace();
			exception = e;
		} finally {
			try { ds.close(); } catch(Exception e) { }
		}
		isRunning = false;
		boolean anyDiscovered;
		if(discovered.size() > 0)
			anyDiscovered = true;
		else
			anyDiscovered = false;
		AnnouncerFinishedEvent evt = new AnnouncerFinishedEvent(anyDiscovered);
		notifyAnnouncerFinishedEventListeners(evt);
	}
	
	private void notifyDiscoveredHostsEventListeners(Host h) {
		DiscoveredHostEvent evt = new DiscoveredHostEvent(h);
		for(int i=0;i<discoveredListeners.size();i++) {
			((DiscoveredHostEventListener)discoveredListeners.get(i)).discoveredHostEvent(evt);
		}
	}
	
	public void addDiscoveredHostsEventListener(DiscoveredHostEventListener l) {
		discoveredListeners.add(l);
	}

	public void addAnnouncerFinishedEventListener(AnnouncerFinishedEventListener l) {
		finishedListeners.add(l);
	}
	
	private void notifyAnnouncerFinishedEventListeners(AnnouncerFinishedEvent evt) {
		for(int i=0;i<finishedListeners.size();i++) {
			((AnnouncerFinishedEventListener)finishedListeners.get(i)).announcerFinishedEvent(evt);
		}
	}
}
