/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote;

import java.io.IOException;
import java.io.Serializable;

import java.util.List;
import java.util.ArrayList;

import walstad.itunesremote.common.*;


/**
 * Methods for controlling the "player" on the
 * remote device
 */
public class Player {

	private SearchResult searchResult = null;
	
	private Connection connection;
	
	private List searchResultListeners = new ArrayList();
	
	public Player(Connection connection) {
		this.connection = connection;
	}
	
	
	private int sendMessage(int code, Serializable data){
		try {
			connection.send(new Message(code,data));
		}catch(IOException e) {
			e.printStackTrace();
			return Common.UNEXPECTED_ERROR;
		}
		return Common.SUCCESS;
	}
	
	public int play(Track t) {
		return sendMessage(Common.PLAY_ID,t);
	}
	
	public int search(SearchQuery sq) {
		return sendMessage(Common.SEARCH_ALL_QUERY,sq);
	}
	
	public int stop() {
		return sendMessage(Common.STOP_ID,null);
	}
	
	public int volumeUp() {
		return sendMessage(Common.VOLUME_UP_ID,null);
	}
	
	public int volumeDown() {
		return sendMessage(Common.VOLUME_DOWN_ID,null);
	}
	
	public int fastForward()  {
		return sendMessage(Common.FAST_FORWARD_ID,null);
	}
	
	public int rewind() {
		return sendMessage(Common.REWIND_ID,null);
	}
	
	public int mute() {
		return sendMessage(Common.MUTE_ID,null);
	}
	
	public int quit() {
		return sendMessage(Common.QUIT_ID,null);
	}
	
	public int nextTrack() {
		return sendMessage(Common.NEXT_TRACK_ID,null);
	}
	
	public int previousTrack() {
		return sendMessage(Common.PREV_TRACK_ID,null);
	}
	
	public int playPause() {
		return sendMessage(Common.PLAY_PAUSE_ID,null);
	}
	
	public int resume() {
		return sendMessage(Common.RESUME_ID,null);
	}
	
	public int getCurrentTrackInfo() {
		return sendMessage(Common.GET_CURRENT_TRACK_INFO_ID,null);
		
	}
	
	public synchronized SearchResult getSearchResult() {
		return searchResult;
	}
	
	public synchronized void setSearchResult(SearchResult sr) {
		searchResult = sr;
		SearchResultChangedEvent evt = new SearchResultChangedEvent(searchResult);
		for(int i=0;i<searchResultListeners.size();i++) {
			((SearchResultChangedListener)searchResultListeners.get(i)).searchResultChanged(evt);
		}
	}
	
	public void addSearchResultChangedActionListener(SearchResultChangedListener list) {
		searchResultListeners.add(list);
	}

	
}
