/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote;


import java.io.IOException;

import walstad.itunesremote.common.*;
import walstad.itunesremote.remote.gui.GuiPasswordErrorDialog;

/**
 * Receives message from the computer and
 * reacts to them
 * 
 * This includes search results, current track info,
 * current volume, play state, etc.
 */
public class Receiver extends Thread {

	private Player player;
	private Connection connection;
	
	private volatile boolean keepRunning = true;
	private Exception error = null;
	
	public Receiver(Player player, Connection connection) {
		this.player = player;
		this.connection = connection;
	}
	
	public void end() {
		keepRunning = false;
	}
	
	public boolean wasError() {
		return (error != null);
	}
	
	public Exception getError() {
		return error;
	}
	
	public void run() {
		while(keepRunning) {
			Message m = null;
			try {
				m = connection.receive();
			}catch(IOException e) {
				e.printStackTrace();
				error = e;
				break;
			}catch(ClassNotFoundException e) {
				e.printStackTrace();
				continue;
			}
			handleMessage(m);
		}
	}
	
	private void handleMessage(Message m) {
		int code = m.getCode();
		switch(code) {
		case Common.SEARCH_RESULTS_HEADER:
			handleSearchResult(m);
			break;
		case Common.BAD_PASSWORD:
		case Common.REQUIRES_PASSWORD:
		    handlePasswordError(m);
			break;
		default:
			throw new UnsupportedOperationException("invalid code");
		}
	}
	
	private int handlePasswordError(Message m) {
	    Main.disconnect();
	    GuiPasswordErrorDialog dialog = new GuiPasswordErrorDialog(m.getCode());
	    dialog.show();
	    return Common.SUCCESS;
	}
	
	private int handleSearchResult(Message m) {
		SearchResult sr = (SearchResult)m.getData();
		player.setSearchResult(sr);
		return Common.SUCCESS;
	}
	
	

}
