/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote.gui;

import java.awt.*;
import java.awt.event.*;


import java.net.InetAddress;

import walstad.itunesremote.common.*;
import walstad.itunesremote.remote.Main;

/**
 * The Gui for adding a new host
 * 
 * @author Chris Walstad
 *
 */
public class GuiAddHostDialog extends Dialog implements ActionListener {
    
    private Button addButton;
    private Button cancelButton;
    private Label nameLabel;
    private Label ipLabel;
    private Label passwordLabel;
    private TextField nameField;
    private TextField ipField;
    private TextField passwordField;
    private Checkbox rememberPasswordCheckBox;
    
    private GuiStartupPanel parent;
    
    public GuiAddHostDialog(GuiStartupPanel parent) {
        super(Main.getMainFrame(),"Add a New Host",true);
        this.parent = parent;
        setupGui();
    }
    
    private void setupGui() {
        
        setSize(200,150);
        setResizable(false);
        
        addButton = new Button("Add Host");
        cancelButton = new Button("Cancel");
        nameLabel = new Label("Host name");
        ipLabel = new Label("IP address");
        passwordLabel = new Label("Password");
        nameField = new TextField();
        ipField = new TextField();
        passwordField = new TextField();
        passwordField.setEchoChar('*');
        rememberPasswordCheckBox = new Checkbox("Remember password",false);
        
        GridBagConstraints gbc = new GridBagConstraints();
        setLayout(new GridBagLayout());
        
        Tools.setGBC(gbc,0,0,1,1,100,0,GridBagConstraints.NONE);
        add(nameLabel,gbc);
        
        Tools.setGBC(gbc,1,0,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(nameField,gbc);
        
        Tools.setGBC(gbc,0,1,1,1,100,0,GridBagConstraints.NONE);
        add(ipLabel,gbc);
        
        Tools.setGBC(gbc,1,1,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(ipField,gbc);
        
        Tools.setGBC(gbc,0,2,1,1,100,0,GridBagConstraints.NONE);
        add(passwordLabel,gbc);
        
        Tools.setGBC(gbc,1,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(passwordField,gbc);
        
        Tools.setGBC(gbc,0,3,2,1,100,0,GridBagConstraints.HORIZONTAL);
        add(rememberPasswordCheckBox,gbc);
        
        Tools.setGBC(gbc,0,4,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(addButton,gbc);
        
        Tools.setGBC(gbc,1,4,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(cancelButton,gbc);
        
        cancelButton.addActionListener(this);
        addButton.addActionListener(this);
        addWindowListener(new DialogWindowListener(this));
    }
    
    private void handleAddButtonActionPerformed(ActionEvent evt) {
		String name = nameField.getText();
		String ip = ipField.getText();
		String password = passwordField.getText();
		boolean rememberPassword = rememberPasswordCheckBox.getState();
		InetAddress addr = null;
		if(ip == null || ip.equals("")) {
			parent.showErrorDialog("Invalid IP","IPs should be of the form x.x.x.x");
			return;
		}
		try {
			addr = InetAddress.getByName(ip);
		}catch(Exception ex) {
			parent.showErrorDialog("Invalid IP", "IPs should be of the form x.x.x.x");
			return;
		}
		if(name == null || name.equals("")) {
			parent.showErrorDialog("Invalid name","You must enter a non-empty description of the host");
			return;
		}
		Host newHost = new Host(name,addr,password,rememberPassword);
		parent.addNewHost(newHost);
		dispose();
    }
    
    public void actionPerformed(ActionEvent evt) {
        if(evt.getSource() == cancelButton) {
            dispose();
        }else if(evt.getSource() == addButton) {
            handleAddButtonActionPerformed(evt);
        }else {
            Debugger.println("Got unrecognized actionPerformed(ActionEvent)");
        }
    }
}
