/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote.gui;

import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Dialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.*;
import java.net.InetAddress;

import walstad.itunesremote.common.Debugger;
import walstad.itunesremote.common.Host;
import walstad.itunesremote.common.Tools;
import walstad.itunesremote.remote.Main;

/**
 * 
 *
 * @author Chris Walstad
 */
public class GuiEditHostDialog extends Dialog implements ActionListener {
    
    private Button applyButton;
    private Button cancelButton;
    private Label nameLabel;
    private Label ipLabel;
    private Label passwordLabel;
    private TextField nameField;
    private TextField ipField;
    private TextField passwordField;
    private Checkbox rememberPasswordCheckBox;
    
    private Host host;
    private GuiStartupPanel parent;
    private int idx;
    
    public GuiEditHostDialog(GuiStartupPanel parent,Host host,  int idx) {
        super(Main.getMainFrame(),"Edit a Host",true);
        this.parent = parent;
        this.host = host;
        this.idx = idx;
        setupGui(host.getHostName(),host.getAddress().getHostAddress(),
                host.getPassword(),host.isRememberPassword());
    }
    
    private void setupGui(String hostName, String ip, String password, 
            boolean rememberPassword) {
        
        setSize(200,150);
        setResizable(false);
        
        applyButton = new Button("Apply");
        cancelButton = new Button("Cancel");
        nameLabel = new Label("Host name");
        ipLabel = new Label("IP address");
        passwordLabel = new Label("Password");
        nameField = new TextField();
        nameField.setText(hostName);
        ipField = new TextField();
        ipField.setText(ip);
        passwordField = new TextField();
        passwordField.setText(password);
        passwordField.setEchoChar('*');
        rememberPasswordCheckBox = new Checkbox("Remember password",false);
        rememberPasswordCheckBox.setState(rememberPassword);
        
        GridBagConstraints gbc = new GridBagConstraints();
        setLayout(new GridBagLayout());
        
        Tools.setGBC(gbc,0,0,1,1,100,0,GridBagConstraints.NONE);
        add(nameLabel,gbc);
        
        Tools.setGBC(gbc,1,0,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(nameField,gbc);
        
        Tools.setGBC(gbc,0,1,1,1,100,0,GridBagConstraints.NONE);
        add(ipLabel,gbc);
        
        Tools.setGBC(gbc,1,1,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(ipField,gbc);
        
        Tools.setGBC(gbc,0,2,1,1,100,0,GridBagConstraints.NONE);
        add(passwordLabel,gbc);
        
        Tools.setGBC(gbc,1,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(passwordField,gbc);
        
        Tools.setGBC(gbc,0,3,2,1,100,0,GridBagConstraints.HORIZONTAL);
        add(rememberPasswordCheckBox,gbc);
        
        Tools.setGBC(gbc,0,4,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(applyButton,gbc);
        
        Tools.setGBC(gbc,1,4,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(cancelButton,gbc);
        
        cancelButton.addActionListener(this);
        applyButton.addActionListener(this);
        addWindowListener(new DialogWindowListener(this));
    }
    
    private void handleApplyButtonActionPerformed(ActionEvent evt) {
		String name = nameField.getText();
		String ip = ipField.getText();
		String password = passwordField.getText();
		boolean rememberPassword = rememberPasswordCheckBox.getState();
		InetAddress addr = null;
		if(ip == null || ip.equals("")) {
			parent.showErrorDialog("Invalid IP","IPs should be of the form x.x.x.x");
			return;
		}
		try {
			addr = InetAddress.getByName(ip);
		}catch(Exception ex) {
			parent.showErrorDialog("Invalid IP", "IPs should be of the form x.x.x.x");
			return;
		}
		if(name == null || name.equals("")) {
			parent.showErrorDialog("Invalid name","You must enter a non-empty description of the host");
			return;
		}
		host.setHostName(name);
		host.setAddress(addr);
		host.setPassword(password);
		host.setRememberPassword(rememberPassword);
		parent.editHost(host, idx);
		dispose();
    }
    
    public void actionPerformed(ActionEvent evt) {
        if(evt.getSource() == cancelButton) {
            dispose();
        }else if(evt.getSource() == applyButton) {
            handleApplyButtonActionPerformed(evt);
        }else {
            Debugger.println("Got unrecognized actionPerformed(ActionEvent)");
        }
    }
    
}
