/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote.gui;

import java.awt.*;
import java.awt.event.*;

import walstad.itunesremote.common.Debugger;
import walstad.itunesremote.common.Tools;
import walstad.itunesremote.remote.Main;


/**
 * @author Chris Walstad
 */
public class GuiGetUserPasswordDialog extends Dialog implements ActionListener {
    
    private volatile boolean isDone = false;
    private String password = null;
    private boolean rememberPassword = false;
    
    private Button cancelButton;
    private Button okButton;
    private TextArea label;
    private TextField passwordField;
    private Checkbox rememberPasswordCheckBox;
    
    public GuiGetUserPasswordDialog() {
        super(Main.getMainFrame(),"Enter a password",true);
        setupGui();
    }
    
    private void setupGui() {
        
        setSize(200,150);
        setResizable(false);
        
        cancelButton = new Button("Cancel");
        okButton = new Button("OK");
        label = new TextArea("The computer you are trying to connect to requires a password.\n" +
                "Please enter one now.",4,10,TextArea.SCROLLBARS_NONE);
        label.setEditable(false);
        passwordField = new TextField();
        passwordField.setEchoChar('*');
        rememberPasswordCheckBox = new Checkbox("Remember password",false);
        
        setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        
        Tools.setGBC(gbc,0,0,2,1,100,100,GridBagConstraints.BOTH);
        add(label,gbc);
        
        Tools.setGBC(gbc,0,1,2,1,100,0,GridBagConstraints.HORIZONTAL);
        add(passwordField,gbc);
        
        Tools.setGBC(gbc,0,2,2,1,100,0,GridBagConstraints.HORIZONTAL);
        add(rememberPasswordCheckBox,gbc);
        
        Tools.setGBC(gbc,0,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(okButton,gbc);
        
        Tools.setGBC(gbc,1,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
        add(cancelButton,gbc);
        
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
        
        addWindowListener(new WindowListener() {
            public void windowActivated(WindowEvent arg0) {}
            public void windowClosed(WindowEvent arg0) {}
            public void windowClosing(WindowEvent arg0) {
                isDone = true;
                dispose();
            }
            public void windowIconified(WindowEvent arg0) {}
            public void windowDeiconified(WindowEvent arg0) {}
            public void windowDeactivated(WindowEvent arg0) {}
            public void windowOpened(WindowEvent arg0) {}
        });
    }
    
    public boolean isDone() {
        return isDone;
    }
    
    public String getPassword() {
        return password;
    }
    
    public boolean rememberPassword() {
        return rememberPassword;
    }
    
    public void actionPerformed(ActionEvent evt) {
        if(evt.getSource() == cancelButton) {
            isDone = true;
            dispose();
        }else if(evt.getSource() == okButton) {
            if(!passwordField.getText().equals("")) {
                password = passwordField.getText();
                if(rememberPasswordCheckBox.getState())
                    rememberPassword = true;
                isDone = true;
                dispose();
            }else {
                label.append("\nPassword cannot be empty.");
            }
        }else {
            Debugger.println("got unrecognized actionPerformed in GuiGetUserPasswordDialog.java");
        }
    }
}
