/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote.gui;

import java.awt.*;
import java.awt.event.*;

import walstad.itunesremote.common.*;
import walstad.itunesremote.remote.Main;
import walstad.itunesremote.remote.Player;
import walstad.itunesremote.remote.SearchResultChangedEvent;
import walstad.itunesremote.remote.SearchResultChangedListener;


/**
 * The main panel for the PersonalJava 1.2 remote gui
 *
 */
public class GuiMainPanel extends Panel implements ActionListener,
							SearchResultChangedListener {

	private TextField searchField = null;
	private Button searchButton = null;
	private List searchResultList = null;
	
	private Button volumeUpButton = null;
	private Button volumeDownButton = null;
	private Button muteButton = null;
	private Button nextTrackButton = null;
	private Button prevTrackButton = null;
	private Button playPauseButton = null;
	private Button fastForwardButton = null;
	private Button rewindButton = null;
	private Button resumeButton = null;
	private Button disconnectButton = null;
	
	private Player player = null;
	
	/**
	 * Create a new main panel
	 */
	public GuiMainPanel() {
		super();
		player = Main.getPlayer();
		setup();
	}
	
	private void setup() {
		
		player.addSearchResultChangedActionListener(this);
		
		setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		searchField = new TextField();
		searchButton = new Button("Search");
		searchResultList = new List();
		searchResultList.setMultipleMode(false);
		
		// buttons
		volumeUpButton = new Button("Up");
		volumeDownButton = new Button("Down");
		muteButton = new Button("Mute");
		nextTrackButton = new Button("Next");
		prevTrackButton = new Button("Prev");
		playPauseButton = new Button("Play");
		fastForwardButton = new Button("Forward");
		rewindButton = new Button("Rewind");
		resumeButton = new Button("Resume");
		disconnectButton = new Button("Disconnect");
		
		// fonts
		Font font = new Font("TimesRoman",Font.PLAIN,11);
		searchResultList.setFont(font);
		volumeUpButton.setFont(font);
		volumeDownButton.setFont(font);
		muteButton.setFont(font);
		nextTrackButton.setFont(font);
		prevTrackButton.setFont(font);
		playPauseButton.setFont(font);
		fastForwardButton.setFont(font);
		rewindButton.setFont(font);
		resumeButton.setFont(font);
		disconnectButton.setFont(font);
		
		Tools.setGBC(gbc,0,0,4,1,100,0,GridBagConstraints.HORIZONTAL);
		add(searchField,gbc);
		
		Tools.setGBC(gbc,4,0,1,1,0,0,GridBagConstraints.NONE);
		add(searchButton,gbc);
		
		Tools.setGBC(gbc,0,1,5,1,100,100,GridBagConstraints.BOTH);
		add(searchResultList,gbc);
		
		Tools.setGBC(gbc,0,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(prevTrackButton,gbc);
		
		Tools.setGBC(gbc,0,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(rewindButton,gbc);
		
		Tools.setGBC(gbc,1,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(playPauseButton,gbc);
		
		Tools.setGBC(gbc,1,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(resumeButton,gbc);
		
		Tools.setGBC(gbc,2,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(nextTrackButton,gbc);
		
		Tools.setGBC(gbc,2,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(fastForwardButton,gbc);
		
		Tools.setGBC(gbc,3,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(volumeUpButton,gbc);
		
		Tools.setGBC(gbc,3,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(volumeDownButton,gbc);
		
		Tools.setGBC(gbc,4,2,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(muteButton,gbc);
		
		Tools.setGBC(gbc,4,3,1,1,100,0,GridBagConstraints.HORIZONTAL);
		add(disconnectButton,gbc);
		
		setBackgroundColors();
		
		searchButton.addActionListener(this);
		searchField.addActionListener(this);
		searchResultList.addActionListener(this);
		
		volumeUpButton.addActionListener(this);
		volumeDownButton.addActionListener(this);
		muteButton.addActionListener(this);
		nextTrackButton.addActionListener(this);
		prevTrackButton.addActionListener(this);
		playPauseButton.addActionListener(this);
		fastForwardButton.addActionListener(this);
		rewindButton.addActionListener(this);
		resumeButton.addActionListener(this);
		disconnectButton.addActionListener(this);
	}
	
	private void setBackgroundColors() {
		Color c = new Color(200,200,200);
		setBackground(c);		
		c = new Color(237,243,254);
		searchResultList.setBackground(c);
	}
	
	
	/**
	 * Handles action events
	 * The following are the current action events this handles:
	 * 1) ActionEvents for searchButton (user clicks to trigger a search)
	 * 2) ActionEvents for searchField (user hits enter to trigger search)
	 * 3) ActionEvents for searchResultList (user double clicks a list item
	 *    to indicate they wish it to be played)
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == searchButton || e.getSource() == searchField) {
			handleSearchActionEvent(e);
		}else if(e.getSource() == searchResultList) {
			handleSearchResultListActionEvent(e);
		}else if(e.getSource() == prevTrackButton) {
			player.previousTrack();
		}else if(e.getSource() == nextTrackButton) {
			player.nextTrack();
		}else if(e.getSource() == playPauseButton) {
			player.playPause();
		}else if(e.getSource() == resumeButton) {
			player.resume();
		}else if(e.getSource() == rewindButton) {
			player.rewind();
		}else if(e.getSource() == fastForwardButton) {
			player.fastForward();
		}else if(e.getSource() == volumeUpButton) {
			player.volumeUp();
		}else if(e.getSource() == volumeDownButton) {
			player.volumeDown();
		}else if(e.getSource() == muteButton) {
			player.mute();
		}else if(e.getSource() == disconnectButton) {
			Main.disconnect();
		}else {
			Debugger.println("Got an invalid ActionEvent in GuiMainPanel.actionPerformed()");
		}
	}
	
	private void handleSearchActionEvent(ActionEvent e) {
		String query = searchField.getText();
		query = query.trim();
		if(query.equals(""))
			return;
		SearchQuery sq = new SearchQuery(query);
		player.search(sq);
	}
	
	private void handleSearchResultListActionEvent(ActionEvent e) {
		Track[] tracks = player.getSearchResult().getTracks();
		int idx = searchResultList.getSelectedIndex();
		player.play(tracks[idx]);
	}
	
	public void searchResultChanged(SearchResultChangedEvent evt) {
		SearchResult sr = evt.getSearchResult();
		searchResultList.removeAll();
		Track[] tracks = sr.getTracks();
		if(tracks == null || tracks.length == 0) {
			searchResultList.add("No results found");
		}else {
			for(int i=0;i<tracks.length;i++) {
				searchResultList.add(tracks[i].getTitle() + ", " + tracks[i].getArtist()
						+ ", " + tracks[i].getAlbum());
			}
		}
	}
}
