/**
 * iTunes Remote
 * Copyright cwalstad 2004,2005
 */

package walstad.itunesremote.remote.gui;

import java.awt.*;
import java.awt.event.*;

import walstad.itunesremote.common.*;
import walstad.itunesremote.remote.Announcer;
import walstad.itunesremote.remote.AnnouncerFinishedEvent;
import walstad.itunesremote.remote.AnnouncerFinishedEventListener;
import walstad.itunesremote.remote.DiscoveredHostEvent;
import walstad.itunesremote.remote.DiscoveredHostEventListener;
import walstad.itunesremote.remote.Main;
import walstad.itunesremote.remote.SavedHosts;


// TODO need to add something that will allow a user to modify saved hosts

/**
 * The panel for the startup screen, connect to discovered devices
 * or saved devices
 */
public class GuiStartupPanel extends Panel implements ActionListener,
								DiscoveredHostEventListener,
								AnnouncerFinishedEventListener {
	
	private Label topLabel;
	
	// Discovering GUI stuff
	private Label discoveredLabel;
	private Button discoverButton;
	private Button stopButton;
	private List discoveredList;
	
	// Saved GUI stuff
	private Label savedHostsLabel;
	private List savedHostsList;
	private Button addNewHostButton;
	private Button removeHostButton;
	private Button editHostButton;
	
	// Non-GUI stuff
	private Announcer announcer;
	private static final int TIMEOUT = 3000;
	private java.util.List discoveredHosts;
	private SavedHosts sh;
	
	public GuiStartupPanel() {
		super(new GridBagLayout());
		discoveredHosts = new java.util.ArrayList();
		setup();
	}
	
	private void setup() {
		sh = SavedHosts.getSavedHosts();
		java.util.List savedHosts = sh.getHosts();
		
		topLabel = new Label("Choose a host to connect to:");
		discoveredLabel = new Label("Discovered hosts:");
		discoverButton = new Button("Discover");
		stopButton = new Button("Stop");
		discoveredList = new List();
		discoveredList.setMultipleMode(false);
		
		savedHostsLabel = new Label("Saved Hosts:");
		savedHostsList = new List();
		savedHostsList.setMultipleMode(false);
		// populate savedHosts list
		if(savedHosts != null) {
			for(int i=0;i<savedHosts.size();i++) {
				savedHostsList.add(((Host)savedHosts.get(i)).toString());
			}
		}
		
		addNewHostButton = new Button("Add New");
		removeHostButton = new Button("Remove");
		editHostButton = new Button("Edit Host");
		
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(new GridBagLayout());
		
		Tools.setGBC(gbc,0,0,6,1,100,0,GridBagConstraints.HORIZONTAL);
		add(topLabel,gbc);
		
		Tools.setGBC(gbc,0,1,4,1,100,0,GridBagConstraints.HORIZONTAL);
		add(discoveredLabel,gbc);
		
		Tools.setGBC(gbc,4,1,1,1,0,0,GridBagConstraints.NONE);
		add(discoverButton,gbc);
		
		Tools.setGBC(gbc,5,1,1,1,0,0,GridBagConstraints.NONE);
		add(stopButton,gbc);
		
		Tools.setGBC(gbc,0,2,6,1,100,100,GridBagConstraints.BOTH);
		add(discoveredList,gbc);
		
		Tools.setGBC(gbc,0,3,6,1,100,0,GridBagConstraints.HORIZONTAL);
		add(savedHostsLabel,gbc);
		
		Tools.setGBC(gbc,0,4,6,1,100,100,GridBagConstraints.BOTH);
		add(savedHostsList,gbc);
		
		Tools.setGBC(gbc,0,5,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(addNewHostButton,gbc);
		
		Tools.setGBC(gbc,2,5,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(editHostButton,gbc);
		
		Tools.setGBC(gbc,4,5,2,1,100,0,GridBagConstraints.HORIZONTAL);
		add(removeHostButton,gbc);
		
		setBackgroundColor();
		
		stopButton.setEnabled(false);
		discoverButton.addActionListener(this);
		discoveredList.addActionListener(this);
		savedHostsList.addActionListener(this);
		stopButton.addActionListener(this);
		addNewHostButton.addActionListener(this);
		removeHostButton.addActionListener(this);
		editHostButton.addActionListener(this);
	}
	
	private void setBackgroundColor() {
		Color c = new Color(200,200,200);
		setBackground(c);
		
		c = new Color(237,243,254);
		discoveredList.setBackground(c);
		savedHostsList.setBackground(c);
	}
	
	private static final String CONNECTION_ERROR_TITLE = "Unable to connect";
	private static final String CONNECTION_ERROR_MESSAGE = "iTunes Remote was unable to connect.  " +
			"Check that the IP is correct and that the host is not already " +
			"connected to a device.";
	
	private void handleDiscoveredListActionPerformed(ActionEvent e) {
		int idx = discoveredList.getSelectedIndex();
		if(idx < 0 || idx >= discoveredHosts.size())
			return;
		Host h = (Host)discoveredHosts.get(idx);
		if(!Main.connect(h)) {
			showErrorDialog(CONNECTION_ERROR_TITLE,CONNECTION_ERROR_MESSAGE);
		}else {
			if(!sh.contains(h)) {
				sh.add(h);
				savedHostsList.add(h.toString());
			}
		}
	}
	
	private void handleSavedHostsListActionPerformed(ActionEvent e) {
		int idx = savedHostsList.getSelectedIndex();
		java.util.List savedHosts = sh.getHosts();
		if(idx < 0 || idx >= savedHosts.size())
			return;
		Host h = (Host)savedHosts.get(idx);
		if(!Main.connect(h)) {
			showErrorDialog(CONNECTION_ERROR_TITLE,CONNECTION_ERROR_MESSAGE);
		}
	}
	
	private synchronized void handleDiscoveredButtonActionEvent(ActionEvent e) {
		stopButton.setEnabled(true);
		discoverButton.setEnabled(false);
		discoveredList.removeAll();
		while(discoveredHosts.size() > 0)
			discoveredHosts.remove(0);
		announcer = new Announcer(TIMEOUT);
		announcer.addDiscoveredHostsEventListener(this);
		announcer.addAnnouncerFinishedEventListener(this);
		announcer.start();
	}
	
	private void handleStopButtonActionPerformed(ActionEvent e) {
		stopButton.setEnabled(false);
		announcer.end();	
	}
	
	public void actionPerformed(ActionEvent e) {
		// TODO need to add protection for if user clicks
		// on selection twice, does making this synchronized do this?
		// I don't think so...
		if(e.getSource() == discoveredList) {
			handleDiscoveredListActionPerformed(e);
		}else if(e.getSource() == savedHostsList) {
			handleSavedHostsListActionPerformed(e);
		}else if(e.getSource() == discoverButton) {
			handleDiscoveredButtonActionEvent(e);
		}else if(e.getSource() == stopButton) {
			handleStopButtonActionPerformed(e);
		}else if(e.getSource() == removeHostButton) {
			handleRemoveHostButtonActionPerformed(e);
		}else if(e.getSource() == addNewHostButton) {
			handleAddNewHostButtonActionPerformed(e);
		}else if(e.getSource() == editHostButton) {
		    handleEditHostButtonActionPerformed(e);
		}else {
			Debugger.println("got invalid event in startuppanel");
		}
	}
	
	private void handleEditHostButtonActionPerformed(ActionEvent e) {
	    int idx = savedHostsList.getSelectedIndex();
	    if(idx < 0 || idx > savedHostsList.getItemCount()) {
	        showErrorDialog("Error","You must select a host to edit.");
	        return;
	    }
	    Host host = sh.getHost(idx);
	    GuiEditHostDialog editHostDialog = new GuiEditHostDialog(this,host,idx);
	    editHostDialog.show();
	}

	private void handleAddNewHostButtonActionPerformed(ActionEvent evt) {
	    GuiAddHostDialog addHostDialog = new GuiAddHostDialog(this);
	    addHostDialog.show();
	}
	
	public boolean isDuplicateHost(Host h) {
	    return sh.contains(h);
	}
	
	public void addNewHost(Host h) {
	    sh.add(h);
		savedHostsList.add(h.toString());
	}
	
	public void showErrorDialog(String title,String msg) {
		final Dialog dialog = new Dialog(Main.getMainFrame(),title,true);
		dialog.setLayout(new GridBagLayout());
		TextArea textArea = new TextArea(msg,4,10,TextArea.SCROLLBARS_NONE);
		textArea.setBackground(dialog.getBackground());
		textArea.setCaretPosition(0);	
		Button okButton = new Button("OK");
		okButton.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
		        dialog.dispose();
		    }
		});
		
		GridBagConstraints gbc = new GridBagConstraints();
		
		Tools.setGBC(gbc,0,0,1,1,100,100,GridBagConstraints.BOTH);
		dialog.add(textArea,gbc);
		Tools.setGBC(gbc,0,1,1,1,100,20,GridBagConstraints.HORIZONTAL);
		dialog.add(okButton,gbc);
		
		dialog.validate();
		dialog.setSize(200,200);
		dialog.addWindowListener(new DialogWindowListener(dialog));
		dialog.setVisible(true);
	}
	
	private void handleRemoveHostButtonActionPerformed(ActionEvent e) {
		int idx = savedHostsList.getSelectedIndex();
		java.util.List savedHosts = sh.getHosts();
		if(idx < 0 || idx >= savedHosts.size())
			return;
		Host h = (Host)savedHosts.get(idx);
		sh.remove(h);
		savedHostsList.remove(idx);
	}

	public synchronized void discoveredHostEvent(DiscoveredHostEvent evt) {
		discoveredHosts.add(evt.getHost());
		discoveredList.add(evt.getHost().toString());
	}
	
	public void announcerFinishedEvent(AnnouncerFinishedEvent evt) {
		discoverButton.setEnabled(true);
		stopButton.setEnabled(false);
		if(!evt.anyDiscovered()) {
			discoveredList.add("No hosts were discovered");
		}
	}

	/**
	 * The host is passed in already modified
	 * @param host
	 */
    public void editHost(Host host, int idx) {
        sh.writeToFile();
        savedHostsList.remove(idx);
        savedHostsList.add(host.toString(),idx);
    }
	
}

class DialogWindowListener implements WindowListener {
	
	Dialog dialog;
	
	public DialogWindowListener(Dialog d) {
		dialog = d;
	}
	
	public void windowOpened(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {
		dialog.dispose();
	}
	public void windowClosed(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowActivated(WindowEvent e) {}
	public void windowDeactivated(WindowEvent e) {}
}

